from .util import Envoy, is_valid
import multiprocessing
from concurrent.futures import ThreadPoolExecutor


class Generic(Envoy):

    def __init__(self, debug=False, token=None, base_url='https://api.botanalytics.co/v1/',
                 callback=None, async = False):
        """
        :param debug: bool
            (default False)
        :param token: str
            Botanalytics token
        :param base_url: str
        :param callback: function
            Error callback function of format (err, reason, payload)
        """
        super(Generic, self).__init__(debug, token, base_url, callback)
        self._inform("Logging enabled for Generic...")
        self.__async = async
        if self.__async:
            self.__number_of_workers = multiprocessing.cpu_count()
            if self.__number_of_workers == 0:
                self.__number_of_workers = 3
            self.__executor_service = ThreadPoolExecutor(max_workers=self.__number_of_workers)
            self._inform("Mode: Async...")

    def log(self, payload):
        """
        :param payload: dict
                 Payload
        :return:
        """
        validation = self.__validate(payload)
        if validation['ok']:
            self._inform('Logging message...')
            self._inform(payload)
            if self.__async:
                self.__executor_service.submit(self._submit, payload)
            else:
                self._submit(payload)
        else:
            self._fail(validation['err'], validation['reason'], payload)

    @staticmethod
    def __validate(payload):
        """
        :param payload:dict
            payload
        :return: dict
            Validation Result
        """

        pv = is_valid(payload, dict, 'payload')
        if not pv['ok']:
            return pv
        pv = is_valid(payload, bool, 'payload', 'is_sender_bot')
        if not pv['ok']:
            return pv
        pv = is_valid(payload, str, 'payload', 'user', 'id')
        if not pv['ok']:
            return pv
        pv = is_valid(payload, str, 'payload', 'user', 'name')
        if not pv['ok']:
            return pv
        pv = is_valid(payload, str, 'payload', 'message', 'text')
        if not pv['ok']:
            return pv
        return is_valid(payload, int, 'payload', 'message', 'timestamp')



from .util import Envoy, is_valid
import multiprocessing
from concurrent.futures import ThreadPoolExecutor
import threading



class AmazonAlexa(Envoy):

    def __init__(self, debug=False, token=None, base_url='https://api.botanalytics.co/v1/',
                 callback=None, async=False):
        """
        :param debug: bool
            (default False)
        :param token: str
            Botanalytics token
        :param base_url: str
        :param callback: function
            Error callback function of format (err, reason, payload)
        """
        super(AmazonAlexa, self).__init__(debug, token, base_url, callback)
        self.__path = 'messages/amazon-alexa/'
        self._inform("Logging enabled for AmazonAlexa...")
        self.__async = async
        if self.__async:
            self.__number_of_workers = multiprocessing.cpu_count()
            if self.__number_of_workers == 0:
                self.__number_of_workers = 3
            self.__executor_service = ThreadPoolExecutor(max_workers=self.__number_of_workers)
            self._inform("Mode: Async...")

    def log(self, req, resp):
        """
        :param req: dict
            User Payload
        :param resp: dict
            Action Payload
        :return:
        """
        validation = self.__validate(req, resp)
        if validation['ok']:
            payload = {'request': req, 'response': resp}
            self._inform('Logging messages...')
            self._inform(payload)
            if self.__async:
                self.__executor_service.submit(self._submit, payload, self.__path)
            else:
                self._submit(payload, self.__path)
        else:
            payload = {'request': req, 'response': resp}
            self._fail(validation['err'], validation['reason'], payload)

    @staticmethod
    def __validate(req, resp):
        """
        :param req:dict
            User Payload
        :param resp: dict
            Skill Payload
        :return: dict
            Validation Result
        """
        pv = is_valid(req, dict, 'request')
        if not pv['ok']:
            return pv
        pv = is_valid(req, dict, 'request', 'request')
        if not pv['ok']:
            return pv
        pv = is_valid(req, str, 'request', 'context', 'System', 'user', 'userId')
        if not pv['ok']:
            return pv
        pv = is_valid(resp, dict, 'response')
        if not pv['ok']:
            return pv
        return is_valid(resp, dict, 'response', 'response')

class AmazonAlexaLambda(Envoy):

    def __init__(self, debug=False, token=None, base_url='https://api.botanalytics.co/v1/',
                 async=False):
        """
        :param debug: bool
            (default False)
        :param token: str
            Botanalytics token
        :param base_url: str

        :param async : bool
            Log async
        """
        super(AmazonAlexaLambda, self).__init__(debug, token, base_url)
        self.__path = 'messages/amazon-alexa/'
        self._inform("Logging enabled for AmazonAlexa...")
        self.__number_of_workers = multiprocessing.cpu_count()
        self.__async = async
        if self.__async:
            self._inform("Mode: Async...")

    def log(self, req, resp):
        """
        :param req: dict
            User Payload
        :param resp: dict
            Action Payload
        :return:
        """
        validation = self.__validate(req, resp)
        if validation['ok']:
            payload = {'request': req, 'response': resp}
            self._inform('Logging messages...')
            self._inform(payload)
            if self.__async:
                t = threading.Thread(target=self._submit, args=(payload, self.__path), daemon=True)
                t.start()
            else:
                self._submit(payload, self.__path)
        else:
            payload = {'request': req, 'response': resp}
            self._fail(validation['err'], validation['reason'], payload)

    @staticmethod
    def __validate(req, resp):
        """
        :param req:dict
            User Payload
        :param resp: dict
            Skill Payload
        :return: dict
            Validation Result
        """
        pv = is_valid(req, dict, 'request')
        if not pv['ok']:
            return pv
        pv = is_valid(req, dict, 'request', 'request')
        if not pv['ok']:
            return pv
        pv = is_valid(req, str, 'request', 'context', 'System', 'user', 'userId')
        if not pv['ok']:
            return pv
        pv = is_valid(resp, dict, 'response')
        if not pv['ok']:
            return pv
        return is_valid(resp, dict, 'response', 'response')




from .util import Envoy, is_valid
from concurrent.futures import ThreadPoolExecutor
import multiprocessing
import time

class FacebookMessenger(Envoy):
    def __init__(self, debug=False, token=None, fb_token=None, base_url='https://api.botanalytics.co/v1/', callback=None, async = False):
        """
        :param debug: bool
            To enable logging
        :param token: str
            Botanalytics token
        :param fb_token: str
            Facebook token (optional)
        :param base_url: str
        :param callback: function
            Error callback of format (err, reason, payload)
        """
        super(FacebookMessenger, self).__init__(debug, token, base_url, callback)
        self.__fb_token = fb_token
        self.__path = "messages/facebook-messenger/"
        self.__profile_path = "facebook-messenger/users/"
        self.__async = async
        self._inform("Debugging enabled for Facebook...")
        if self.__async:
            self.__number_of_workers = multiprocessing.cpu_count()
            if self.__number_of_workers == 0:
                self.__number_of_workers = 2
            self.__executor_service = ThreadPoolExecutor(max_workers=self.__number_of_workers)
            self._inform("Mode: Async...")

    def log_incoming(self, message):
        """
        Logs incoming message
        :param message: dict
            Incoming message
        :return:
        """
        validation = self.__validate(message, "Incoming message")
        if validation['ok']:
            payload = {
                'recipient': None,
                'timestamp': int(time.time()*1000),
                'message': message
            }
            self._inform("Logging incoming message...")
            self._inform(message)
            if self.__async:
                self.__executor_service.submit(self._submit, payload, self.__path)
            else:
                self._submit(payload, self.__path)
        else:
            self._fail(validation['err'], validation['reason'], message)

    def log_outgoing(self, message, sender_id):
        """
        Logs outgoing message
        :param message: dict
            Outgoing message payload
        :param sender_id: str
            Sender id
        :return:
        """
        validation = self.__validate_outgoing(message, sender_id)
        if validation['ok']:
            payload = {
                'recipient': sender_id,
                'timestamp': int(time.time()*1000),
                'message': message,
                'fb_token': self.__fb_token
            }
            self._inform("Logging outgoing message...")
            self._inform(message)
            if self.__async:
                self.__executor_service.submit(self._submit, payload, self.__path)
            else:
                self._submit(payload, self.__path)
        else:
            self._fail(validation['err'], validation['reason'], message)

    def log_user_profile(self, message):
        """
        Logs user profile
        :param message: dict
            User profile
        :return:
        """
        validation = self.__validate(message, "User profile")
        if validation['ok']:
            self._inform("Logging user profile message...")
            self._inform(message)
            if self.__async:
                self.__executor_service.submit(self._submit, message, self.__profile_path)
            else:
                self._submit(message, self.__profile_path)
        else:
            self._fail(validation['err'], validation['reason'], message)

    @staticmethod
    def __validate(payload, name):
        return is_valid(payload, dict, name)

    @staticmethod
    def __validate_outgoing(message, sender_id):
        mv = is_valid(message, dict, "Outgoing message")
        if not mv['ok']:
            return mv
        return is_valid(sender_id, str, "Sender id")


from __future__ import print_function
import requests
import re
import json

ip_middle_octet = u"(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5]))"
ip_last_octet = u"(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))"
# validators url regex
regex = re.compile(
    u"^"
    u"(?:(?:https?|ftp)://)"
    u"(?:[-a-z\u00a1-\uffff0-9._~%!$&'()*+,;=:]+"
    u"(?::[-a-z0-9._~%!$&'()*+,;=:]*)?@)?"
    u"(?:"
    u"(?P<private_ip>"
    u"(?:(?:10|127)" + ip_middle_octet + u"{2}" + ip_last_octet + u")|"
    u"(?:(?:169\.254|192\.168)" + ip_middle_octet + ip_last_octet + u")|"
    u"(?:172\.(?:1[6-9]|2\d|3[0-1])" + ip_middle_octet + ip_last_octet + u"))"
    u"|"
    u"(?P<private_host>"
    u"(?:localhost))"
    u"|"
    u"(?P<public_ip>"
    u"(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])"
    u"" + ip_middle_octet + u"{2}"
    u"" + ip_last_octet + u")"
    u"|"
    u"(?:(?:[a-z\u00a1-\uffff0-9]-?)*[a-z\u00a1-\uffff0-9]+)"
    u"(?:\.(?:[a-z\u00a1-\uffff0-9]-?)*[a-z\u00a1-\uffff0-9]+)*"
    u"(?:\.(?:[a-z\u00a1-\uffff]{2,}))"
    u")"
    u"(?::\d{2,5})?"
    u"(?:/[-a-z\u00a1-\uffff0-9._~%!$&'()*+,;=:@/]*)?"
    u"(?:\?\S*)?"
    u"(?:#\S*)?"
    u"$",
    re.UNICODE | re.IGNORECASE
)

pattern = re.compile(regex)


# validators url
def valid_url(url, public=False):
    result = pattern.match(url)

    if not public:
        return result

    return result and not any(
        (result.groupdict().get(key) for key in ('private_ip', 'private_host'))
    )


def is_valid(base_payload, t, name, *args):
    if base_payload is None:
        return {
            'ok' : False,
            'reason' : 'None is not accepted',
            'err' : BaseException('Payload is None!')
        }
    if args:
        key_path = ''
        temp = None
        try:
            for arg in args:
                if temp:
                    temp = temp[arg]
                    key_path += arg
                else:
                    temp = base_payload[arg]
                    key_path += arg
        except KeyError as e:
            return {
                'ok': False,
                'reason': 'Field does not exist!',
                'err': BaseException('Expected field <{}> can not be found in {}'.format(key_path+'.'+str(e)[1:-1] if key_path else str(e)[1:-1], name))
            }

        if isinstance(temp, t):
            return {'ok': True}
        else:
            return {
                'ok': False,
                'reason': 'Unexpected format!',
                'err': BaseException('Expected format for {} is {}, found {}'.format(name, t.__name__, type(temp).__name__))
            }
    else:
        if isinstance(base_payload, t):
            return {'ok': True}
        else:
            return {'ok': False,
                    'reason': 'Unexpected format!',
                    'err': BaseException('Expected format for {} is {}, found {}'.format(name, t.__name__, type(base_payload).__name__))
                    }


class Envoy(object):
    def __init__(self, debug=False, token=None, base_url='https://api.botanalytics.co/v1/',
                 callback=None):
        """
        :param debug: bool
            Enables logging (optional)
        :param token: str
            Botanalytics token (required)
        :param base_url: str
            Endpoint
        :param callback: function
            Error callback (optional)
        """
        self.__debug = debug
        self.__token = token
        self.__base_url = base_url
        self.__callback = callback
        if not valid_url(self.__base_url):
            raise ValueError("Url is not valid!")
        if not self.__token:
            raise ValueError("Token is not provided!")

    # Debug
    def _inform(self, payload):
        """
        log.debug
        :param payload: str or dict
        :return:
        """
        if self.__debug:
            if isinstance(payload, str):
                print("[Botanalytics Debug]: %s" % payload)
                return
            if isinstance(payload, dict):
                print("[Botanalytics Debug]: %s" % json.dumps(payload))

    # Error
    def _fail(self, error, reason, payload=None):
        """
        log.error
        :param error: BaseException
        :param reason: str
        :param payload: dict
        :return:
        """
        if self.__callback is not None:
            self.__callback(error, reason, payload)
        else:
            print("[Botanalytics Error]: {}, {}...\n {}".format(reason, str(error), payload))

    # Sends messages
    def _submit(self, parchment, destination="messages/generic/", word=None):
        try:
            path = self.__base_url + destination
            headers = {'Authorization': 'Token ' + self.__token}
            r = requests.post(path, headers=headers, json=parchment)
            if r.status_code == 200 or r.status_code == 201:
                if word is not None:
                    self._inform(word)
                else:
                    self._inform('Successfully logged message(s)...')
                return True
            else:
                response = r.json()
                self._fail(BaseException('Message(s) can not be logged!'), response['error_message'], parchment)
                return False
        except BaseException as e:
            self._fail(e, '', parchment)
            return False



from setuptools import setup
readme = open("README.md","r").read()

setup(
    name = 'botanalytics',
    version = '0.2.2',
    description = 'Conversational analytics & engagement tool for chatbots',
    long_description = readme,
    url = 'https://github.com/Botanalyticsco/botanalytics-python',
    author = 'Beyhan Esen',
    author_email = 'tech@botanalytics.co',
    license = 'MIT',
    classifiers = [
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3'
    ],
    keywords = 'botanalytics bot bots analytics chatbot chatbots conversational facebook google amazon ibm voice actions lex alexa messsenger assistant watson',
    install_requires = [
        'requests',
        'futures; python_version == "2.7"'
    ],
    packages = ['botanalytics']
)



from .util import Envoy, is_valid
from concurrent.futures import ThreadPoolExecutor
import threading
import multiprocessing

class GoogleAssistant(Envoy):

    def __init__(self, debug=False, token=None, base_url='https://api.botanalytics.co/v1/',
                 callback=None, async = False):
        """
        :param debug: bool
            (default False)
        :param token: str
            Botanalytics token
        :param base_url: str
        :param callback: function
            Error callback function of format (err, reason, payload)
        """
        super(GoogleAssistant, self).__init__(debug, token, base_url, callback)
        self.__path = 'messages/google-assistant/'
        self._inform("Logging enabled for GoogleAssistant...")
        self.__async = async
        if self.__async:
            self.__number_of_workers = multiprocessing.cpu_count()
            if self.__number_of_workers == 0:
                self.__number_of_workers = 3
            self.__executor_service = ThreadPoolExecutor(max_workers=self.__number_of_workers)
            self._inform("Mode: Async...")

    def log(self, req, resp):
        """
        :param req: dict
            User Payload
        :param resp: dict
            Action Payload
        :return:
        """
        validation = self.__validate(req, resp)
        if validation['ok']:
            payload = {'request': req, 'response': resp}
            self._inform('Logging messages...')
            self._inform(payload)
            if self.__async:
                self.__executor_service.submit(self._submit, payload, self.__path)
            else:
                self._submit(payload, self.__path)
        else:
            payload = {'request': req, 'response': resp}
            self._fail(validation['err'], validation['reason'], payload)

    @staticmethod
    def __validate(req, resp):
        """
        :param req:dict
            User Payload
        :param resp: dict
            Action Payload
        :return: dict
            Validation Result
        """

        pv = is_valid(req, dict, 'request')
        if not pv['ok']:
            return pv
        pv = is_valid(req, str, 'request', 'user', 'userId')
        if not pv['ok']:
            return pv
        pv = is_valid(req, str, 'request', 'conversation', 'conversationId')
        if not pv['ok']:
            return pv
        pv = is_valid(resp, dict, 'response')
        if not pv['ok']:
            return pv
        try:
            if resp['expectUserResponse']:
                return is_valid(resp, list, 'response', 'expectedInputs')

            else:
                return is_valid(resp, dict, 'response', 'finalResponse')

        except BaseException as e:
            return {'ok': False,
                    'reason': 'Invalid response payload, expected fields are not found!',
                    'err': e
                    }

class GoogleAssistantCloudFunctions(Envoy):

    def __init__(self, debug=False, token=None, base_url='https://api.botanalytics.co/v1/',
                 async=False):
        """
        :param debug: bool
            (default False)
        :param token: str
            Botanalytics token
        :param base_url: str
        :param async: function
            Error callback function of format (err, reason, payload)
        """
        super(GoogleAssistantCloudFunctions, self).__init__(debug, token, base_url)
        self.__path = 'messages/google-assistant/'
        self._inform("Logging enabled for GoogleAssistant...")
        self.__async = async
        if self.__async:
            self._inform("Mode: Async...")

    def log(self, req, resp):
        """
        :param req: dict
            User Payload
        :param resp: dict
            Action Payload
        :return:
        """
        validation = self.__validate(req, resp)
        if validation['ok']:
            payload = {'request': req, 'response': resp}
            self._inform('Logging messages...')
            self._inform(payload)
            if self.__async:
                t = threading.Thread(target=self._submit, args=(payload, self.__path), daemon=True)
                t.start()
            else:
                self._submit(payload, self.__path)
        else:
            payload = {'request': req, 'response': resp}
            self._fail(validation['err'], validation['reason'], payload)

    @staticmethod
    def __validate(req, resp):
        """
        :param req:dict
            User Payload
        :param resp: dict
            Action Payload
        :return: dict
            Validation Result
        """

        pv = is_valid(req, dict, 'request')
        if not pv['ok']:
            return pv
        pv = is_valid(req, str, 'request', 'user', 'userId')
        if not pv['ok']:
            return pv
        pv = is_valid(req, str, 'request', 'conversation', 'conversationId')
        if not pv['ok']:
            return pv
        pv = is_valid(resp, dict, 'response')
        if not pv['ok']:
            return pv
        try:
            if resp['expectUserResponse']:
                return is_valid(resp, list, 'response', 'expectedInputs')

            else:
                return is_valid(resp, dict, 'response', 'finalResponse')

        except BaseException as e:
            return {'ok': False,
                    'reason': 'Invalid response payload, expected fields are not found!',
                    'err': e
                    }




from .util import Envoy, is_valid
from concurrent.futures import ThreadPoolExecutor
import multiprocessing
import time
import requests
import threading


class SlackRTMApi(Envoy):
    def __init__(self, debug=False, token=None, slack_client_instance=None, base_url='https://api.botanalytics.co/v1/',
                 callback=None, async=False):
        """
        :param debug: bool
            Enables logging
        :param token: str
            Botanalytics token
        :param slack_client_instance:
            Slack client instance
        :param base_url:

        :param callback:
        """
        super(SlackRTMApi, self).__init__(debug=debug, token=token, base_url=base_url, callback=callback)
        self.__path = 'messages/slack/'
        self.__initialize_path = 'bots/slack/initialize/'
        self.__activeUserId = None
        self.__activeTeamId = None
        if slack_client_instance is None:
            raise ValueError('Slack instance is not provided!')
        self.__slack_client = slack_client_instance
        self._inform('Logging enabled for SlackRTMApi...')
        self.__async = async
        if self.__async:
            self.__executor_service = ThreadPoolExecutor(max_workers=2)
            self.__executor_service.submit(self.__initialize)
            self._inform("Mode: Async...")
        else:
            t = threading.Thread(target=self.__initialize, daemon=False)
            t.start()

    def __initialize(self):
        wait_interval = 2
        update_interval = 3600
        form = {'token': self.__slack_client.token}
        while True:
            try:
                r = requests.post('https://slack.com/api/rtm.start', data=form)
                if r.status_code == 200 or r.status_code == 201:
                    data = r.json()
                    if not data['ok']:
                        time.sleep(wait_interval)
                        wait_interval += 3
                        continue
                    self.__activeUserId = data['self']['id']
                    self.__activeTeamId = data['team']['id']
                    if not self._submit(data, self.__initialize_path, "Successfully updated bot '{}' info...".format(data['self']['name'])):
                        time.sleep(wait_interval)
                        wait_interval += 3
                    else:
                        time.sleep(update_interval)
                        wait_interval = 2
            except BaseException as e:
                self._fail(e, 'Unknown reason')

    def log_incoming(self, payload):
        """
        :param payload: dict
            Message payload

        :return:
        """
        if self.__activeTeamId is None or self.__activeUserId is None:
            self._fail(BaseException('team and bot'), 'Missing info')
            return

        validation = self.__validate(payload)
        if validation['ok']:
            self._inform('Logging message...')
            payload['is_bot'] = False
            payload['team'] = self.__activeTeamId
            self._inform(payload)
            if self.__async:
                self.__executor_service.submit(self._submit, payload, self.__path)
            else:
                self._submit(payload,self.__path)
        else:
            self._inform("Message does not contain 'type' field. Ignoring...")
            self._inform(payload)

    def log_outgoing(self, channel, message, thread=None, reply_broadcast=None, msg_payload=None):

        if self.__activeTeamId is None or self.__activeUserId is None:
            self._fail(BaseException('team and bot'), 'Missing info')
            return

        if msg_payload is not None:
            if not isinstance(msg_payload, dict):
                self._fail(BaseException('Expected format for msg_payload is dict found {}'.format(type(msg_payload).__name__)),
                           'Unexpected payload format!')
                return
            msg = msg_payload.copy()
            validation = self.__validate(msg)
            if validation['ok']:
                msg['is_bot'] = True
                msg['ts'] = str(int(time.time() * 1000) / 1000)
                msg['team'] = self.__activeTeamId
                msg['user'] = self.__activeUserId
                self._inform('Logging message...')
                self._inform(msg)
                if self.__async:
                    self.__executor_service.submit(self._submit, message, self.__path)
                else:
                    self._submit(message, self.__path)
            else:
                self._inform("Message does not contain 'type' field. Ignoring...")
                self._inform(msg_payload)
                return
        found_channel = self.__slack_client.server.channels.find(channel)
        channel_id = found_channel.id if found_channel else channel
        payload = {"type": "message", "channel": channel_id, "text": message}
        if thread is not None:
            payload["thread_ts"] = thread
            if reply_broadcast:
                payload['reply_broadcast'] = True
        payload['is_bot'] = True
        payload['ts'] = str(int(time.time()*1000)/1000)
        payload['team'] = self.__activeTeamId
        payload['user'] = self.__activeUserId
        self._inform('Logging message...')
        self._inform(payload)
        if self.__async:
            self.__executor_service.submit(self._submit, payload, self.__path)
        else:
            self._submit(payload, self.__path)

    @staticmethod
    def __validate(payload):
        return is_valid(payload, str, 'msg_payload', 'type')


class SlackEventApi(Envoy):
    def __init__(self, debug=False, token=None, slack_token=None, base_url='https://api.botanalytics.co/v1/',
                 callback=None, async = False):
        """
        :param debug: bool
            Enables logging
        :param token: str
            Botanalytics token
        :param slack_token:
            Slack token
        :param base_url:

        :param callback:
        """
        super(SlackEventApi, self).__init__(debug=debug, token=token, base_url=base_url, callback=callback)
        self.__path = 'messages/slack/event/'
        self.__initialize_path = 'bots/slack/initialize/'
        self.__interactive_path = 'messages/slack/interactive/'
        if slack_token is None:
            raise ValueError('Slack token is not provided!')
        self.__slack_token = slack_token
        self._inform('Logging enabled for SlackEventApi...')
        self.__async = async
        if self.__async:
            self.__number_of_workers = multiprocessing.cpu_count()
            if self.__number_of_workers == 0:
                # +1 for initialize update
                self.__number_of_workers = 3
            self.__executor_service = ThreadPoolExecutor(max_workers=self.__number_of_workers)
            self.__executor_service.submit(self.__initialize)
            self._inform("Mode: Async...")
        else:
            t = threading.Thread(target=self.__initialize, daemon=False)
            t.start()

    def __initialize(self):
        wait_interval = 2
        update_interval = 3600
        form = {'token': self.__slack_token}
        while True:
            try:
                r = requests.post('https://slack.com/api/rtm.start', data=form)
                if r.status_code == 200 or r.status_code == 201:
                    data = r.json()
                    if not data['ok']:
                        time.sleep(wait_interval)
                        wait_interval += 3
                        continue
                    self.__activeUserId = data['self']['id']
                    self.__activeTeamId = data['team']['id']
                    if not self._submit(data, self.__initialize_path, "Successfully updated bot '{}' info...".format(data['self']['name'])):
                        time.sleep(wait_interval)
                        wait_interval += 3
                    else:
                        time.sleep(update_interval)
                        wait_interval = 2
            except BaseException as e:
                self._fail(e, 'Unknown.')

    def log(self, payload):
        """
        :param payload: dict
            Message payload

        :return:
        """
        try:
            if payload['challenge']:
                return
        except BaseException:
            pass

        validation = self.__validate(payload)
        if validation['ok']:
            self._inform('Logging message...')
            self._inform(payload)
            type_name = payload['type']
            if type_name == "interactive_message":
                if self.__async:
                    self.__executor_service.submit(self._submit, payload, self.__interactive_path)
                else:
                    self._submit(payload, self.__interactive_path)
            elif type_name == "event_callback":
                if self.__async:
                    self.__executor_service.submit(self._submit, payload, self.__path)
                else:
                    self._submit(payload, self.__path)
            else:
                self._fail(
                    BaseException('Expected types, [interactive_message, event_callback] but found {}'.format(type_name)),
                    'New event type, contact us < tech@botanalytics.co >',
                    payload
                )
        else:
            self._inform("Message does not contain 'type' field. Ignoring...")

    @staticmethod
    def __validate(payload):
        return is_valid(payload, str, 'payload', 'type')
